globals [nbirths ndeaths crude-birth crude-death growth-rate i x1 x2 birthrate-hist deathrate-hist birth-moving-avg death-moving-avg alpha]
breed [males male]
breed [females female]
turtles-own [age] ;; for keeping track of when the turtle is ready
                     ;; to reproduce and when it will die

to setup
  clear-all
  setup-patches
  setup-turtles
  reset-ticks
end

to setup-patches
  ask patches [ set pcolor green ]
end

to setup-turtles
  ;create-turtles 1000    ;; uses the value of the number slider to create turtles
  create-males (initial-pop / 2) [set color blue set age random 100 set shape "person"]
  create-females (initial-pop / 2) [set color pink set age random 100 set shape "person"] 
  ask turtles [ setxy random-xcor random-ycor ]
end

to go
  if count turtles = 0 [ stop ]  ;; stop after 500 ticks
  set ndeaths 0
  set nbirths 0
  move-turtles
  inf-mort
  inc-age
  check-death
  reproduce
  
  ifelse count turtles != 0
  [set growth-rate  ((nbirths - ndeaths) / count turtles) * 100]
  [set growth-rate 0]
  
  tick
  ;ifelse count turtles != 0
end  

to move-turtles
  ask turtles [
    right random 360
    forward 1
  ]
end

to inc-age
  ask turtles [
   set age age + 1
  ifelse age < 100
  [ask males[set color scale-color blue age 100 0]
    ask females [set color scale-color pink age 100 0]
  ]
  [set color black]
  ] 
end

to reproduce 
  if ticks = 0 [set birthrate-hist [0 0 0 0 0 0 0 0 0 0]]
  ask females with [age >= 15 and age <= 45]
  [if random 1000 < (total-fertility-rate / (45 - 15)) * 1000 [
    set nbirths nbirths + 1
    ifelse random 2 = 0
    [ hatch-females 1 [set age 0 set color white set shape "person"] ]
    [ hatch-males 1 [set age 0 set color white set shape "person"] ]
  ]  
  ]
  ifelse count turtles  != 0
  [set crude-birth (nbirths / count turtles) * 1000]
  [set crude-birth 0]
  set birthrate-hist but-first birthrate-hist
  set birthrate-hist lput crude-birth birthrate-hist
  set birth-moving-avg mean birthrate-hist
end

to inf-mort
  ;print infant-mortality
  ;print count turtles with [age = 0]
  ask turtles with [ age = 0]
  [
   if (random 1000 < infant-mortality)[ 
     set ndeaths ndeaths + 1
     die] 
  ]
end

to check-death
  if ticks = 0 [set deathrate-hist [0 0 0 0 0 0 0 0 0 0]]
  set alpha (0.085 * 0.5 / (exp(0.085 * life-expectancy) - 1))
  ask turtles [
    ;Mapping life expectancy to probability of death through the 50% point on the Gompertz cummulative density function
     ;if random 1000 < ( 1 - exp(-(0.693) * (1 - exp(0.085 * age)) / (1 - exp(0.085 * life-expectancy)))) * 1000 [
      ;if random 1000 < ( 1 - ( exp ( ( 1 - exp(0.085 * age)) / ( 1 - exp(0.085 * life-expectancy)) * -0.693))) * 1000 [  
      if random 1000 < (alpha * exp(0.085 * age)) * 1000 [
      set ndeaths ndeaths + 1
      die
    ]
  ]
  ifelse count turtles  != 0 
  [set crude-death (ndeaths / count turtles) * 1000]
  [set crude-death 0]
  set deathrate-hist but-first deathrate-hist
  set deathrate-hist lput crude-death deathrate-hist
  set death-moving-avg mean deathrate-hist
end
@#$#@#$#@
GRAPHICS-WINDOW
148
10
617
445
16
14
13.9310345
1
10
1
1
1
0
1
1
1
-16
16
-14
14
1
1
1
ticks
30.0

BUTTON
10
23
73
56
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
9
63
72
96
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

PLOT
930
10
1230
293
Age Structure Diagram
age
number
0.0
100.0
-50.0
50.0
true
false
"" ""
PENS
"pen-1" 1.0 1 -16777216 true "" "clear-plot\nset i 0\nrepeat 100 [\n     set x1 -1 * (count males with [age = i])\n     set-plot-pen-color blue\n     plotxy i x1\n     plot-pen-up\n     set-plot-pen-color pink\n     set x2 (count females with [age = i] )\n     plotxy i x2\n     set i i + 1\n     \n]"

PLOT
621
10
925
292
Total Population vs. Time
time
population
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"pen-2" 1.0 0 -16777216 true "" "plot count turtles"

MONITOR
1082
360
1182
405
crude birth rate
crude-birth
2
1
11

MONITOR
1082
405
1182
450
crude death rate
crude-death
2
1
11

PLOT
621
294
1076
524
Crude Birth and Death Rates [per 1000 pop.]
time
NIL
0.0
10.0
0.0
20.0
true
true
"" ""
PENS
"birth rate moving average" 1.0 0 -15575016 true "" "plot birth-moving-avg"
"death rate moving average" 1.0 0 -10873583 true "" "plot death-moving-avg"

MONITOR
1082
450
1182
495
growth rate [%]
growth-rate
2
1
11

SLIDER
0
230
145
263
total-fertility-rate
total-fertility-rate
0
5
5
.1
1
NIL
HORIZONTAL

SLIDER
0
265
145
298
infant-mortality
infant-mortality
0
300
300
1
1
NIL
HORIZONTAL

INPUTBOX
10
165
80
225
initial-pop
500
1
0
Number

MONITOR
1080
315
1180
360
Current Population
count turtles
0
1
11

SLIDER
0
300
145
333
life-expectancy
life-expectancy
40
100
100
5
1
years
HORIZONTAL

TEXTBOX
35
131
80
156
Inputs
15
0.0
1

@#$#@#$#@
## WHAT IS IT?

This model simulates a human population over time given:
-a total fertility rate (TFR): the average number of children a woman would have over her lifetime
-an infant mortality rate: for every 1000 births, how many children die before their first birthday?
-a life expectancy: the age for which a person has a 50% chance of dying before that age (ignoring infant mortality in this case)

The agents ("turtles") in the model represent humans, and are described by two properties: their sex and their age.  

## HOW IT WORKS

Every "tick" of the model represents one year. Every year the following happen:
- Infant mortality: The model makes a list of all the agents with age 0. Each infant is assigned a random number, and if this is below a threshold set by the infant-mortality slider, the infant is killed.
-Increment age: All turtles add 1 year to their age.
-Death: Similar to the process for infant mortality, but instead of just infants, all agents draw a random number. This number is compared to the value of a function which provides the probability of death given a person's age and the value of life-expectancy.
-Reproduction: A list of all females of child-bearing age is compiled. Again, a random number is selected for each and compared to a probability set by the total-fertility slider. For example, if the TFR is 3.0, on average a woman will have 3 children in her lifetime. If the length of time she can have children is 30 years, 3 children/30 years = 
0.1 children per year, or a 10% chance of having a child in a given year.
-Plots and outputs are updated with the latest information from that year. 

## HOW TO USE IT

Press the "Setup" button to load the model with an initial population. By default, this creates 500 people (set in the initial-pop box) with approximately equal numbers of males and females (blue and pink in the "world" box, respectively) with a random distribution of ages from 0 to 100. Note, this is NOT A REALISTIC SETUP for ages in the real world -- there are not as many 100 year-olds as babies. The model will start to correct this in the first few years of running, however.

The other parameters for the simulation are set by the sliders on the lefthand side. Their effects are described above in detail, but but broadly, total-fertility determines how many new babies are created, infant-mortality determines how many of them die before age 1, and life-expectancy determines how the probability of death increases with age.

Press "Go" to start the model running with the parameters set by the sliders. When adjusting the sliders, it's often easier to pause the model by clicking Go again and then continuing after making adjustments.

In the world box (center), the agents wander randomly on a green background. Their pink and blue colors are adjusted to start as a light tint when they're young and gradually shade darker as they age. Looking at the world can be useful sometimes, but when there are a lot of people it can cause the model to run slower. Unchecking the box labeled "view updates" at top center hides the world and can help it to run faster.

Of the three plots, the top left shows the total population with time. The top right is an age structure diagram, essentially a histogram of how many people in the model have ages within each 5 year range (0-4, 5-9, etc.). Females and males are plotted with bars extending up and down, respectively. The bottom graph shows a moving average of the crude birth and death rates. That is, every year the model counts how many births and deaths occurred and scales them to a standard population of 1000. Then the model averages these rates over the previous 10 years and plots this value. This reduces the amount of jitter in the rates, since we're looking at a relatively small population. Also note that in the first 10 years of the run, the rates are filled with 0 values, so this graph will also be unrealistic at the very beginning. The numberical outputs show the total population value, the year's crude birth and death rates, and the growth rate percent, calculated by (crude birth rate - crude death rate)/10.

## THINGS TO NOTICE

How does the age structure diagram change with time? At what ages are most of the deaths occurring? How does this compare with the life-expectancy slider? Compare times when the total population is increasing or decreasing with the values of the crude birth rate and crude death rate.

## THINGS TO TRY

Watch the age structure diagram for high and low values of TFR with high and low values of life expectancy. Simulate a baby boom by raising the TFR for a few years and then returning it to something lower. Look up real values for various countries in the CIA World Factbook (https://www.cia.gov/library/publications/the-world-factbook/) for instance, and see how well you can model them. For a given life expectancy, can you find what values of TFR lead to an exponentially increasing population vs. a population that crashes to zero?  

## EXTENDING THE MODEL

Try adapting to have different life expectancies for men and women. Mortality is modeled using a Gompertz Law function depending only on age, but it could have an age-independent component, too (Gompertz-Makeham). The model assumes population only changes through births and deaths -- immigration/emigration could be added to make it more realistic.  

## NETLOGO FEATURES

The NetLogo histogram function doesn't allow for downward-extending bars, so the age structure diagram required more coding by hand.


## CREDITS AND REFERENCES

Adapted from the model in NetLogo Tutorial #3 http://ccl.northwestern.edu/netlogo/docs/tutorial3.html
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
